﻿
using System;
using System.Drawing;
using System.Windows.Forms;

/*          Polynomial interpolation by the use of Givens transformations      */
/*                              www.mosismath.com                              */


namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        public const int order = 6;      // number of reference points
        public const int datapoints = 1000;  // number of datapoints for the interpolation
        public const double deltaX = 0.5; // sample time 

        public TextBox[,] tbm;
        public TextBox[] tbs;
        public TextBox[] tbx;

        public double[,] m = new double[order, order];
        public double[] y = new double[order];
        public double[] x = new double[order];
        public double[] t = new double[order];
        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];
        double[] y_k = new double[order];
        double[] x_k = new double[order];


       /* public class TMatrix
        {
            public int size;
            public double[,] a;
            public double[] y;
            public double[] x;
            public double[] t;

            public TMatrix(int sizeM)
            {
                size = sizeM;
                a = new double[size, size];
                y = new double[size];
                x = new double[size];
            }

            public void MultMatrix(TMatrix m1)
            {
                int i, j, k;
                double[,] al = new double[size, size];
                double[] yl = new double[size];
                for (i = 0; i < size; i++)
                {
                    yl[i] = 0.0;
                    for (j = 0; j < size; j++)
                    {
                        yl[i] = yl[i] + (y[j] * m1.a[i, j]);
                        al[i, j] = 0.0;
                        for (k = 0; k < size; k++)
                        {
                            al[i, j] = al[i, j] + (a[k, j] * m1.a[i, k]);
                        }
                    }
                }
                for (i = 0; i < size; i++)
                {
                    y[i] = yl[i];
                    for (j = 0; j < size; j++)
                    {
                        a[i, j] = al[i, j];
                    }
                }
            }

            public void MultMatrix(TMatrix m1, int p, int q)
            {
                int i, j, k;
                double[,] al = new double[size, size];
                double[] yl = new double[size];
                for (i = 0; i < size; i++)
                {
                    if ((i == p) || (i == q))
                    {
                        yl[i] = 0.0;
                        for (j = 0; j < size; j++)
                        {
                            yl[i] = yl[i] + (y[j] * m1.a[i, j]);
                            al[i, j] = 0.0;
                            for (k = 0; k < size; k++)
                            {
                                al[i, j] = al[i, j] + (a[k, j] * m1.a[i, k]);
                            }
                        }
                    }
                    else
                    {
                        yl[i] = y[i];
                        for (j = 0; j < size; j++)
                        {
                            al[i, j] = a[i, j];
                        }
                    }
                }
                for (i = 0; i < size; i++)
                {
                    y[i] = yl[i];
                    for (j = 0; j < size; j++)
                    {
                        a[i, j] = al[i, j];
                    }
                }
            }
        }

        TMatrix m;
        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];
        double[] y_k = new double[order];
        double[] x_k = new double[order];
 
        public class TGauss
        {
            public bool calcError;
            public int maxOrder = order;
            TMatrix m;
            TMatrix r;

            public void CalcR(int p, int q)
            {
                int i, j;
                double radius;
                radius = Math.Sqrt(m.a[p, p] * m.a[p, p] + m.a[q, p] * m.a[q, p]);
                for (i = 0; i < m.size; i++)
                {
                    for (j = 0; j < m.size; j++)
                    {
                        r.a[i, j] = 0.0;
                    }
                    r.a[i, i] = 1.0;
                }
                if (radius != 0.0)
                {
                    if (m.a[p, p] == 0.0)
                    {
                        r.a[p, p] = 0.0;
                        r.a[p, q] = 1.0;
                    }
                    else
                    {
                        r.a[p, p] = m.a[p, p] / radius;
                        r.a[p, q] = m.a[q, p] / radius;
                    }
                    r.a[q, q] = r.a[p, p];
                    r.a[q, p] = -r.a[p, q];
                }

            }

            public void Mult(int p, int q)
            {
                m.MultMatrix(r, p, q);
            }



            void SwitchRows(int n)
            {
                double tempD;
                int i, j;
                for (i = n; i <= maxOrder - 2; i++)
                {
                    for (j = 0; j <= maxOrder - 1; j++)
                    {
                        tempD = m.a[i, j];
                        m.a[i, j] = m.a[i + 1, j];
                        m.a[i + 1, j] = tempD;
                    }
                    tempD = m.y[i];
                    m.y[i] = m.y[i + 1];
                    m.y[i + 1] = tempD; 
                }
            }


            public TGauss(int size, TMatrix mi)
            {
                maxOrder = size;
                r = new TMatrix(size);
                m = mi;
            }

            public bool Eliminate()
            {
                int i, k, l;
                calcError = false;
                for (k = 0; k <= maxOrder - 2; k++)
                {
                    for (i = k; i <= maxOrder - 2; i++)
                    {
                        if (Math.Abs(m.a[i+1, i]) < 1e-8)
                        {
                            SwitchRows(i+1);
                        }
                        if (m.a[i + 1, k] != 0.0)
                        {
                            for (l = k + 1; l <= maxOrder - 1; l++)
                            {
                                if (!calcError)
                                {
                                    m.a[i + 1, l] = m.a[i + 1, l] * m.a[k, k] - m.a[k, l] * m.a[i + 1, k];
                                    if (m.a[i + 1, l] > 10E260)
                                    {
                                        m.a[i + 1, k] = 0;  // range overflow 
                                        calcError = true;
                                    }
                                }
                            }
                            m.y[i + 1] = m.y[i + 1] * m.a[k, k] - m.y[k] * m.a[i + 1, k];
                            m.a[i + 1, k] = 0;
                        }
                    }
                }
                return !calcError;
            }

            public void Solve()
            {
                int k, l;
                for (k = maxOrder - 1; k >= 0; k--)
                {
                    for (l = maxOrder - 1; l >= k; l--)
                    {
                        m.y[k] = m.y[k] - m.x[l] * m.a[k, l];
                    }
                    if (m.a[k, k] != 0)
                        m.x[k] = m.y[k] / m.a[k, k];
                    else
                        m.x[k] = 0;
                }
            }

            public void SolveGivens()
            {
                int i, k;
                for (i = 0; i < maxOrder; i++)
                {
                    for (k = i + 1; k < maxOrder; k++)
                    {
                        if (m.a[k, i] != 0.0)
                        {
                            CalcR(i, k);
                            m.MultMatrix(r, i, k);
                        }
                    }
                }
            }
        }

        TGauss gauss;*/

        
        public MainWin()
        {
            int i;
           // m = new TMatrix(order);
            //gauss = new TGauss(order, m);
            tbs = new TextBox[10];
            tbx = new TextBox[10];

            InitializeComponent();

            tbs[0] = textBoxY1; tbs[1] = textBoxY2; tbs[2] = textBoxY3; tbs[3] = textBoxY4; tbs[4] = textBoxY5;
            tbs[5] = textBoxY6; tbs[6] = textBoxY7; tbs[7] = textBoxY8; tbs[8] = textBoxY9; tbs[9] = textBoxY10;

            tbx[0] = textBoxX1; tbx[1] = textBoxX2; tbx[2] = textBoxX3; tbx[3] = textBoxX4; tbx[4] = textBoxX5;
            tbx[5] = textBoxX6; tbx[6] = textBoxX7; tbx[7] = textBoxX8; tbx[8] = textBoxX9; tbx[9] = textBoxX10;
 
            // some sample values for y
             y_k[0] = 0.5;
             y_k[1] = 11.7;
             y_k[2] = 14.8;
             y_k[3] = 4.0;
             y_k[4] = 2.2;
             y_k[5] = 0.2;

            // timestamp to y value
            for (i = 1; i <= order; i++)
            {
                x_k[i - 1] = deltaX * (double)i;
            }

            if (order < 10)
            {
                for (i = 0; i < order; i++)
                {
                    tbs[i].Text = Convert.ToString(y_k[i]);
                    tbx[i].Text = Convert.ToString(x_k[i]);

                }
                for (i = order; i < 10; i++)
                {
                    tbs[i].Visible = false;
                    tbx[i].Visible = false;
                }
            }
        }

        public void Interpolate(int maxPoints, double maxTime)
        {
            int i, k;
            for (i = 0; i < maxPoints; i++)
            {
                yp[i] = 0;
                xp[i] = (double)(i) * maxTime / (double)maxPoints;
                for (k = 0; k < order; k++)
                {   
                    yp[i] = yp[i] + x[k] * Math.Pow( xp[i], (double)(k));
                }
            }
        }

        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                scalefactor = pGraph.Height / (maxVal) / 1.1;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(xp[j] * pGraph.Width / (double)maxTime);
                        p2.X = Convert.ToInt32(xp[j + 1] * pGraph.Width / (double)maxTime);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        g.DrawLine(bluePen, p1, p2);
                    }
                }
                for (j = 0; j < order; j++) // mark reference points
                {
                    p1.X = Convert.ToInt32(x_k[j] * pGraph.Width / (double)maxTime);
                    if (drawCeroline)
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    else
                        p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                }
            }
            g.Dispose();
        }


        private void button1_Click(object sender, EventArgs e)
        {
            int i, k;
            if (order < 10)
            {
                for (i = 0; i < order; i++)
                {
                    if (tbs[i].Text != "")
                    {
                        y_k[i] = Convert.ToDouble(tbs[i].Text);
                        x_k[i] = Convert.ToDouble(tbx[i].Text);
                    }
                    else
                        y_k[i] = 0.0;
                    x[i] = 0.0;
                }
            }
            // fill matrix upside down
            for (i = 0; i < order; i++)
            {
                y[i] = y_k[order - 1 - i]; 
                for (k = 0; k < order; k++)
                {
                    m[i, k] = Math.Pow(x_k[order - 1 - i], (double)(k));
                }   
            }

            TGivens givens = new TGivens(order, m, y);
            givens.RunGivens();
            x = givens.Solve();
            Interpolate(datapoints, x_k[order - 1]);
            DrawGraph(datapoints, x_k[0], x_k[order - 1], true);
          
        }

        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, x_k[0], x_k[order - 1], false);
        }  
    }
}
